#import "../Dialog2.h"
#import "../TMDCommand.h"
#import "../OptionParser.h"

/*
echo '{alertStyle = warning; buttonTitles = ('OK'); messageTitle = 'test'; informativeText = 'Testing';}' | "$DIALOG" alert

"$DIALOG" help alert
"$DIALOG" alert -sc -t "test"
"$DIALOG" alert -s critical -m "FOOL!" -t "test" -1 foo -2 bar -3 baz
"$DIALOG" alert -e -s critical -m "FOOL!" -t "test" -1 foo -2 bar -3 baz
"$DIALOG" alert -e -1 O -2 A -3 B -t BLA -m FOO  -i "/Library/Desktop Pictures/Black & White/Lightning.jpg"
"$DIALOG" alert -h "Error" -c#FF0000
"$DIALOG" alert -e -m "Error" -c#FF000044 
*/

// =========
// = Alert =
// =========

@interface TMDAlertCommand : TMDCommand
{
}
@end

@implementation TMDAlertCommand

int sheetReturn;

+ (void)load
{
	[super registerObject:[self new] forCommand:@"alert"];
}

static option_t const expectedOptions[] =
{
	{ "m", "message",		option_t::required_argument,	option_t::string,	"Message title."},
	{ "t", "text",			option_t::required_argument,	option_t::string,	"Informative text for the alert."},
	{ "s", "alert-style",	option_t::required_argument,	option_t::string,	"One of warning, critical or informational (the default)."},
	{ "1", "button1",		option_t::required_argument,	option_t::string,	"Button 1 label."},
	{ "2", "button2",		option_t::required_argument,	option_t::string,	"Button 2 label."},
	{ "3", "button3",		option_t::required_argument,	option_t::string,	"Button 3 label."},
	{ "e", "sheet",			option_t::no_argument,			option_t::none,		"If given displays as sheet."},
	{ "i", "icon",			option_t::required_argument,	option_t::string,	"File path to an image."},
	{ "h", "header",		option_t::required_argument,	option_t::string,	"Window title (not as sheet)."},
	{ "c", "color",			option_t::required_argument,	option_t::string,	"Background color as #RRGGBBAA (Alpha value only for sheets)."},
};

- (void)handleCommand:(CLIProxy*)proxy
{
	// NSFileHandle* fh = [options objectForKey:@"stderr"];

	SetOptionTemplate(proxy, expectedOptions);
	NSAlertStyle	alertStyle        = NSInformationalAlertStyle;
	NSString		*alertStyleString = [proxy valueForOption:@"alert-style"];
	NSDictionary	*resultDict       = nil;

	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	id documentWindow = nil;
	BOOL modal = YES;
	
	if([alertStyleString hasPrefix:@"w"])
	{
		alertStyle = NSWarningAlertStyle;
	}
	else if([alertStyleString hasPrefix:@"c"])
	{
		alertStyle = NSCriticalAlertStyle;
	}
	else if([alertStyleString hasPrefix:@"i"])
	{
		alertStyle = NSInformationalAlertStyle;
	}
	[alert setAlertStyle:alertStyle];
	
	if([proxy valueForOption:@"message"])
		[alert setMessageText:[proxy valueForOption:@"message"]];
	if([proxy valueForOption:@"text"])
		[alert setInformativeText:[proxy valueForOption:@"text"]];
	
	// Setup icon
	if([proxy valueForOption:@"icon"])
		[alert setIcon:[[NSImage alloc] initWithContentsOfFile:[proxy valueForOption:@"icon"]]];
	
	// Setup buttons
	if ([proxy valueForOption:@"button1"])
		[alert addButtonWithTitle:[proxy valueForOption:@"button1"]];
	if ([proxy valueForOption:@"button2"])
		[alert addButtonWithTitle:[proxy valueForOption:@"button2"]];
	if ([proxy valueForOption:@"button3"])
		[alert addButtonWithTitle:[proxy valueForOption:@"button3"]];
	
	if([[alert buttons] count] == 0)
		[alert addButtonWithTitle:@"OK"];

	// Setup the window's title
	if([proxy valueForOption:@"header"])
		[[alert window] setTitle:[proxy valueForOption:@"header"]];
	
	// Check display as sheet
	if([proxy valueForOption:@"sheet"])
	{
		enumerate([NSApp windows], id item)
		if([item isKindOfClass:NSClassFromString(@"NSWindow")])
			if([item isKeyWindow])
				documentWindow = item;
		modal = (documentWindow == nil) ? YES : NO;
	}

	// Setup the background color
	if([proxy valueForOption:@"color"])
	{
		unsigned int red = 0, green = 0, blue = 0, alpha = 0xFF;
		sscanf([[proxy valueForOption:@"color"] UTF8String], "#%02x%02x%02x%02x", &red, &green, &blue, &alpha);
		if(modal)
			alpha = 0xFF;
		[[alert window] setBackgroundColor:[NSColor colorWithCalibratedRed:red/255.0f green:green/255.0f blue:blue/255.0f alpha:alpha/255.0f]];
	}
	
	// Show the alert
	// as a modal window
	if(modal)
	{
		int alertResult = ([alert runModal] - NSAlertFirstButtonReturn);
		resultDict = [NSDictionary dictionaryWithObject:[NSNumber numberWithInt:alertResult] forKey:@"buttonClicked"];
	}
	// as a sheet bound to the current window
	else
	{
		[alert beginSheetModalForWindow:documentWindow 
						  modalDelegate:self
						 didEndSelector:@selector(mySheetDidEnd:returnCode:contextInfo:) 
							contextInfo:NULL];
		[NSApp runModalForWindow:[alert window]];
		resultDict = [NSDictionary dictionaryWithObject:[NSNumber numberWithInt:(sheetReturn - NSAlertFirstButtonReturn)] forKey:@"buttonClicked"];
	}
	[TMDCommand writePropertyList:resultDict toFileHandle:[proxy outputHandle]];
}

-(id)mySheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[NSApp endSheet:sheet];
	sheetReturn = returnCode;
	[NSApp stopModal];
	return nil;
}


- (NSString *)commandDescription
{
	return @"Show an alert box.";
}

- (NSString *)usageForInvocation:(NSString *)invocation;
{
	return [NSString stringWithFormat:@"%@ «options»\n\nOptions:\n%@", invocation, GetOptionList(expectedOptions)];
}
@end
